@@ -4,7 +4,6 @@
 
 package org.qtproject.qt.android;
 
-import android.content.Context;
 import android.graphics.Rect;
 import android.os.Build;
 import android.os.Bundle;
@@ -29,10 +28,6 @@ class QtAccessibilityDelegate extends View.AccessibilityDelegate
     // all low positive ints should be fine.
     static final int INVALID_ID = 333; // half evil
 
-    // The platform might ask for the class implementing the "view".
-    // Pretend to be an inner class of the QtSurface.
-    private static final String DEFAULT_CLASS_NAME = "$VirtualChild";
-
     private View m_view = null;
     private AccessibilityManager m_manager;
     private QtLayout m_layout;
@@ -66,8 +61,10 @@ class QtAccessibilityDelegate extends View.AccessibilityDelegate
 
     void initLayoutAccessibility(QtLayout layout)
     {
-        if (m_layout == null)
+        if (layout == null) {
             Log.w(TAG, "Unable to initialize the accessibility delegate with a null layout");
+            return;
+        }
 
         m_layout = layout;
 
@@ -245,10 +242,10 @@ class QtAccessibilityDelegate extends View.AccessibilityDelegate
             }
 
             final AccessibilityEvent event =
-                    AccessibilityEvent.obtain(AccessibilityEvent.TYPE_ANNOUNCEMENT);
+                    obtainAccessibilityEvent(AccessibilityEvent.TYPE_ANNOUNCEMENT);
 
             event.setEnabled(true);
-            event.setClassName(m_view.getClass().getName() + DEFAULT_CLASS_NAME);
+            event.setClassName(getNodeForVirtualViewId(viewId).getClassName());
 
             event.setContentDescription(value);
 
@@ -265,6 +262,37 @@ class QtAccessibilityDelegate extends View.AccessibilityDelegate
         });
     }
 
+    void notifyDescriptionOrNameChanged(int viewId, String value)
+    {
+        if (viewId == m_focusedVirtualViewId)
+            notifyValueChanged(viewId, value);
+    }
+
+    void notifyAnnouncementEvent(int viewId, String message)
+    {
+        QtNative.runAction(() -> {
+            if (m_view == null)
+                return;
+
+            if (viewId == INVALID_ID) {
+                Log.w(TAG, "notifyAnnouncementEvent() for invalid view");
+                return;
+            }
+
+            if (!m_manager.isEnabled()) {
+                Log.w(TAG, "notifyAnnouncementEvent for disabled AccessibilityManager");
+                return;
+            }
+
+            final AccessibilityEvent event =
+                    obtainAccessibilityEvent(AccessibilityEvent.TYPE_ANNOUNCEMENT);
+            event.getText().add(message);
+            event.setClassName(getNodeForVirtualViewId(viewId).getClassName());
+            event.setPackageName(m_view.getContext().getPackageName());
+            sendAccessibilityEvent(event);
+        });
+    }
+
     void sendEventForVirtualViewId(int virtualViewId, int eventType)
     {
         final AccessibilityEvent event = getEventForVirtualViewId(virtualViewId, eventType);
@@ -321,10 +349,10 @@ class QtAccessibilityDelegate extends View.AccessibilityDelegate
         if (m_layout == null || m_layout.getChildCount() == 0)
             return null;
 
-        final AccessibilityEvent event = AccessibilityEvent.obtain(eventType);
+        final AccessibilityEvent event = obtainAccessibilityEvent(eventType);
 
         event.setEnabled(true);
-        event.setClassName(m_view.getClass().getName() + DEFAULT_CLASS_NAME);
+        event.setClassName(getNodeForVirtualViewId(virtualViewId).getClassName());
 
         event.setContentDescription(QtNativeAccessibility.descriptionForAccessibleObject(virtualViewId));
         if (event.getText().isEmpty() && TextUtils.isEmpty(event.getContentDescription()))
@@ -352,12 +380,12 @@ class QtAccessibilityDelegate extends View.AccessibilityDelegate
     private AccessibilityNodeInfo getNodeForView()
     {
         if (m_view == null || m_layout == null)
-            return AccessibilityNodeInfo.obtain();
+            return obtainAccessibilityNodeInfo();
 
         // Since we don't want the parent to be focusable, but we can't remove
         // actions from a node, copy over the necessary fields.
-        final AccessibilityNodeInfo result = AccessibilityNodeInfo.obtain(m_view);
-        final AccessibilityNodeInfo source = AccessibilityNodeInfo.obtain(m_view);
+        final AccessibilityNodeInfo result = obtainAccessibilityNodeInfo(m_view);
+        final AccessibilityNodeInfo source = obtainAccessibilityNodeInfo(m_view);
         m_view.onInitializeAccessibilityNodeInfo(source);
 
         // Get the actual position on screen, taking the status bar into account.
@@ -367,8 +395,8 @@ class QtAccessibilityDelegate extends View.AccessibilityDelegate
 
         // Copy over parent and screen bounds.
         final Rect m_tempParentRect = new Rect();
-        source.getBoundsInParent(m_tempParentRect);
-        result.setBoundsInParent(m_tempParentRect);
+        getBoundsInParent(source, m_tempParentRect);
+        setBoundsInParent(result, m_tempParentRect);
 
         final Rect m_tempScreenRect = new Rect();
         source.getBoundsInScreen(m_tempScreenRect);
@@ -416,11 +444,10 @@ class QtAccessibilityDelegate extends View.AccessibilityDelegate
     private AccessibilityNodeInfo getNodeForVirtualViewId(int virtualViewId)
     {
         if (m_view == null || m_layout == null)
-            return AccessibilityNodeInfo.obtain();
+            return obtainAccessibilityNodeInfo();
 
-        final AccessibilityNodeInfo node = AccessibilityNodeInfo.obtain();
+        final AccessibilityNodeInfo node = obtainAccessibilityNodeInfo();
 
-        node.setClassName(m_view.getClass().getName() + DEFAULT_CLASS_NAME);
         node.setPackageName(m_view.getContext().getPackageName());
 
         if (m_layout.getChildCount() == 0 || !QtNativeAccessibility.populateNode(virtualViewId, node)) {
@@ -444,7 +471,7 @@ class QtAccessibilityDelegate extends View.AccessibilityDelegate
 
         Rect parentScreenRect = QtNativeAccessibility.screenRect(parentId);
         screenRect.offset(-parentScreenRect.left, -parentScreenRect.top);
-        node.setBoundsInParent(screenRect);
+        setBoundsInParent(node, screenRect);
 
         // Manage internal accessibility focus state.
         if (m_focusedVirtualViewId == virtualViewId) {
@@ -459,11 +486,7 @@ class QtAccessibilityDelegate extends View.AccessibilityDelegate
         for (int id : ids)
             node.addChild(m_view, id);
         if (node.isScrollable()) {
-            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
-                node.setCollectionInfo(new CollectionInfo(ids.length, 1, false));
-            } else {
-                node.setCollectionInfo(CollectionInfo.obtain(ids.length, 1, false));
-            }
+            setCollectionInfo(node, ids.length, 1, false);
         }
 
         return node;
@@ -491,17 +514,6 @@ class QtAccessibilityDelegate extends View.AccessibilityDelegate
             boolean handled = false;
             //Log.i(TAG, "PERFORM ACTION: " + action + " on " + virtualViewId);
             switch (action) {
-                case AccessibilityNodeInfo.ACTION_ACCESSIBILITY_FOCUS:
-                    // Only handle the FOCUS action if it's placing focus on
-                    // a different view that was previously focused.
-                    if (m_focusedVirtualViewId != virtualViewId) {
-                        m_focusedVirtualViewId = virtualViewId;
-                        m_view.invalidate();
-                        sendEventForVirtualViewId(virtualViewId,
-                                AccessibilityEvent.TYPE_VIEW_ACCESSIBILITY_FOCUSED);
-                        handled = true;
-                    }
-                    break;
                 case AccessibilityNodeInfo.ACTION_CLEAR_ACCESSIBILITY_FOCUS:
                     if (m_focusedVirtualViewId == virtualViewId) {
                         m_focusedVirtualViewId = INVALID_ID;
@@ -541,17 +553,69 @@ class QtAccessibilityDelegate extends View.AccessibilityDelegate
             if (success)
                 sendEventForVirtualViewId(virtualViewId, AccessibilityEvent.TYPE_VIEW_CLICKED);
             break;
+        case AccessibilityNodeInfo.ACTION_ACCESSIBILITY_FOCUS:
+            if (m_focusedVirtualViewId != virtualViewId) {
+                success = QtNativeAccessibility.focusAction(virtualViewId);
+                if (!success) {
+                    notifyObjectFocus(virtualViewId);
+                    success = true;
+                }
+            }
+            break;
         case AccessibilityNodeInfo.ACTION_SCROLL_FORWARD:
             success = QtNativeAccessibility.scrollForward(virtualViewId);
-            if (success)
-                sendEventForVirtualViewId(virtualViewId, AccessibilityEvent.TYPE_VIEW_SCROLLED);
             break;
         case AccessibilityNodeInfo.ACTION_SCROLL_BACKWARD:
             success = QtNativeAccessibility.scrollBackward(virtualViewId);
-            if (success)
-                sendEventForVirtualViewId(virtualViewId, AccessibilityEvent.TYPE_VIEW_SCROLLED);
             break;
         }
         return success;
     }
+
+    @SuppressWarnings("deprecation")
+    private AccessibilityEvent obtainAccessibilityEvent(int eventType) {
+        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
+            return new AccessibilityEvent(eventType);
+        } else {
+            return AccessibilityEvent.obtain(eventType);
+        }
+    }
+
+    @SuppressWarnings("deprecation")
+    private AccessibilityNodeInfo obtainAccessibilityNodeInfo() {
+        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
+            return new AccessibilityNodeInfo();
+        } else {
+            return AccessibilityNodeInfo.obtain();
+        }
+    }
+
+    @SuppressWarnings("deprecation")
+    private AccessibilityNodeInfo obtainAccessibilityNodeInfo(View source) {
+        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
+            return new AccessibilityNodeInfo(source);
+        } else {
+            return AccessibilityNodeInfo.obtain(source);
+        }
+    }
+
+    @SuppressWarnings("deprecation")
+    private void getBoundsInParent(AccessibilityNodeInfo node, Rect outBounds) {
+        node.getBoundsInParent(outBounds);
+    }
+
+    @SuppressWarnings("deprecation")
+    private void setBoundsInParent(AccessibilityNodeInfo node, Rect bounds) {
+        node.setBoundsInParent(bounds);
+    }
+
+    @SuppressWarnings("deprecation")
+    private void setCollectionInfo(AccessibilityNodeInfo node, int rowCount, int columnCount,
+                                   boolean hierarchical) {
+        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
+            node.setCollectionInfo(new CollectionInfo(rowCount, columnCount, hierarchical));
+        } else {
+            node.setCollectionInfo(CollectionInfo.obtain(rowCount, columnCount, hierarchical));
+        }
+    }
 }
