@@ -10,6 +10,13 @@
 #include <QtCore/qdebug.h>
 #include <QtCore/qcoreapplication.h>
 #include <QtCore/qoperatingsystemversion.h>
+#include <QtGui/qcolorspace.h>
+
+#if defined(Q_OS_MACOS)
+# include <AppKit/AppKit.h>
+#elif defined(QT_PLATFORM_UIKIT)
+# include <UIKit/UIKit.h>
+#endif
 
 QT_USE_NAMESPACE
 
@@ -54,17 +61,6 @@ CGImageRef qt_mac_toCGImage(const QImage &inImage)
     return inImage.convertToFormat(QImage::Format_ARGB32_Premultiplied).toCGImage();
 }
 
-CGImageRef qt_mac_toCGImageMask(const QImage &image)
-{
-    static const auto deleter = [](void *image, const void *, size_t) { delete static_cast<QImage *>(image); };
-    QCFType<CGDataProviderRef> dataProvider =
-            CGDataProviderCreateWithData(new QImage(image), image.bits(),
-                                                    image.sizeInBytes(), deleter);
-
-    return CGImageMaskCreate(image.width(), image.height(), 8, image.depth(),
-                              image.bytesPerLine(), dataProvider, NULL, false);
-}
-
 void qt_mac_drawCGImage(CGContextRef inContext, const CGRect *inBounds, CGImageRef inImage)
 {
     CGContextSaveGState( inContext );
@@ -85,6 +81,11 @@ QImage qt_mac_toQImage(CGImageRef image)
     CGRect rect = CGRectMake(0, 0, w, h);
     QMacCGContext ctx(&ret);
     qt_mac_drawCGImage(ctx, &rect, image);
+
+    CGColorSpaceRef colorSpace = CGImageGetColorSpace(image);
+    QCFType<CFDataRef> iccData = CGColorSpaceCopyICCData(colorSpace);
+    ret.setColorSpace(QColorSpace::fromIccProfile(QByteArray::fromRawCFData(iccData)));
+
     return ret;
 }
 
@@ -118,22 +119,29 @@ QT_END_NAMESPACE
 
 + (instancetype)imageFromQIcon:(const QIcon &)icon
 {
-    return [NSImage imageFromQIcon:icon withSize:0];
+    return [NSImage imageFromQIcon:icon withSize:QSize()];
 }
 
-+ (instancetype)imageFromQIcon:(const QIcon &)icon withSize:(int)size
++ (instancetype)imageFromQIcon:(const QIcon &)icon withSize:(const QSize &)size
+{
+    return [NSImage imageFromQIcon:icon withSize:size withMode:QIcon::Normal withState:QIcon::Off];
+}
+
++ (instancetype)imageFromQIcon:(const QIcon &)icon withSize:(const QSize &)size
+                    withMode:(QIcon::Mode)mode withState:(QIcon::State)state
+
 {
     if (icon.isNull())
         return nil;
 
     auto availableSizes = icon.availableSizes();
-    if (availableSizes.isEmpty() && size > 0)
-        availableSizes << QSize(size, size);
+    if (availableSizes.isEmpty() && !size.isNull())
+        availableSizes << size;
 
     auto nsImage = [[[NSImage alloc] initWithSize:NSZeroSize] autorelease];
 
     for (QSize size : std::as_const(availableSizes)) {
-        QImage image = icon.pixmap(size).toImage();
+        const QImage image = icon.pixmap(size, mode, state).toImage();
         if (image.isNull())
             continue;
 
@@ -144,6 +152,9 @@ QT_END_NAMESPACE
         auto *imageRep = [[NSBitmapImageRep alloc] initWithCGImage:cgImage];
         imageRep.size = image.deviceIndependentSize().toCGSize();
         [nsImage addRepresentation:[imageRep autorelease]];
+        // Match behavior of loading icns files, where the NSImage size
+        // reflects the largest representation.
+        nsImage.size = imageRep.size;
     }
 
     if (!nsImage.representations.count)
@@ -151,8 +162,10 @@ QT_END_NAMESPACE
 
     [nsImage setTemplate:icon.isMask()];
 
-    if (size)
-        nsImage.size = CGSizeMake(size, size);
+    if (!size.isNull()) {
+        auto imageSize = QSizeF::fromCGSize(nsImage.size);
+        nsImage.size = imageSize.scaled(size, Qt::KeepAspectRatio).toCGSize();
+    }
 
     return nsImage;
 }
@@ -185,7 +198,7 @@ QPixmap qt_mac_toQPixmap(const NSImage *image, const QSizeF &size)
 
 #endif // Q_OS_MACOS
 
-#ifdef Q_OS_IOS
+#ifdef QT_PLATFORM_UIKIT
 
 QImage qt_mac_toQImage(const UIImage *image, QSizeF size)
 {
@@ -202,7 +215,7 @@ QImage qt_mac_toQImage(const UIImage *image, QSizeF size)
     return ret;
 }
 
-#endif // Q_OS_IOS
+#endif // QT_PLATFORM_UIKIT
 
 // ---------------------- Colors and Brushes ----------------------
 
@@ -228,6 +241,9 @@ QColor qt_mac_toQColor(CGColorRef color)
 #ifdef Q_OS_MACOS
 QColor qt_mac_toQColor(const NSColor *color)
 {
+    if (!color)
+        return QColor();
+
     QColor qtColor;
     switch (color.type) {
     case NSColorTypeComponentBased: {
