@@ -1,5 +1,6 @@
 // Copyright (C) 2022 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qsqlquery.h"
 
@@ -7,6 +8,7 @@
 
 #include "qatomic.h"
 #include "qdebug.h"
+#include "qloggingcategory.h"
 #include "qsqlrecord.h"
 #include "qsqlresult.h"
 #include "qsqldriver.h"
@@ -19,6 +21,8 @@
 
 QT_BEGIN_NAMESPACE
 
+Q_STATIC_LOGGING_CATEGORY(lcSqlQuery, "qt.sql.qsqlquery")
+
 class QSqlQueryPrivate
 {
 public:
@@ -213,11 +217,12 @@ QSqlQuery::~QSqlQuery()
         delete d;
 }
 
-#if QT_DEPRECATED_SINCE(6, 2)
+#if QT_REMOVAL_QT7_DEPRECATED_SINCE(6, 2)
 /*!
     Constructs a copy of \a other.
 
-    \deprecated QSqlQuery cannot be meaningfully copied. Prepared
+    \deprecated [6.2] QSqlQuery cannot be meaningfully copied, and
+    therefore will no longer be copiable in Qt 7. Prepared
     statements, bound values and so on will not work correctly, depending
     on your database driver (for instance, changing the copy will affect
     the original). Treat QSqlQuery as a move-only type instead.
@@ -232,7 +237,8 @@ QSqlQuery::QSqlQuery(const QSqlQuery& other)
 /*!
     Assigns \a other to this object.
 
-    \deprecated QSqlQuery cannot be meaningfully copied. Prepared
+    \deprecated [6.2] QSqlQuery cannot be meaningfully copied, and
+    therefore will no longer be copiable in Qt 7. Prepared
     statements, bound values and so on will not work correctly, depending
     on your database driver (for instance, changing the copy will affect
     the original). Treat QSqlQuery as a move-only type instead.
@@ -260,8 +266,7 @@ QSqlQuery& QSqlQuery::operator=(const QSqlQuery& other)
 /*!
     \fn void QSqlQuery::swap(QSqlQuery &other) noexcept
     \since 6.2
-    Swaps \a other to this object. This operation is very
-    fast and never fails.
+    \memberswap{query}
 */
 
 /*!
@@ -272,7 +277,7 @@ static void qInit(QSqlQuery *q, const QString& query, const QSqlDatabase &db)
     QSqlDatabase database = db;
     if (!database.isValid()) {
         database =
-                QSqlDatabase::database(QLatin1StringView(QSqlDatabase::defaultConnection), false);
+                QSqlDatabase::database(QSqlDatabase::defaultConnectionName(), false);
     }
     if (database.isValid())
         *q = QSqlQuery(database.driver()->createResult());
@@ -332,14 +337,16 @@ bool QSqlQuery::isNull(int field) const
     returns isNull(int index) for the corresponding field index.
 
     This overload is less efficient than \l{QSqlQuery::}{isNull()}
-*/
 
-bool QSqlQuery::isNull(const QString &name) const
+    \note In Qt versions prior to 6.8, this function took QString, not
+    QAnyStringView.
+*/
+bool QSqlQuery::isNull(QAnyStringView name) const
 {
     qsizetype index = d->sqlResult->record().indexOf(name);
     if (index > -1)
         return isNull(index);
-    qWarning("QSqlQuery::isNull: unknown field name '%s'", qPrintable(name));
+    qCWarning(lcSqlQuery, "QSqlQuery::isNull: unknown field name '%ls'", qUtf16Printable(name.toString()));
     return true;
 }
 
@@ -375,7 +382,7 @@ bool QSqlQuery::exec(const QString& query)
     t.start();
 #endif
     if (!driver()) {
-        qWarning("QSqlQuery::exec: called before driver has been set up");
+        qCWarning(lcSqlQuery, "QSqlQuery::exec: called before driver has been set up");
         return false;
     }
     if (d->ref.loadRelaxed() != 1) {
@@ -392,19 +399,20 @@ bool QSqlQuery::exec(const QString& query)
     }
     d->sqlResult->setQuery(query.trimmed());
     if (!driver()->isOpen() || driver()->isOpenError()) {
-        qWarning("QSqlQuery::exec: database not open");
+        qCWarning(lcSqlQuery, "QSqlQuery::exec: database not open");
         return false;
     }
     if (query.isEmpty()) {
-        qWarning("QSqlQuery::exec: empty query");
+        qCWarning(lcSqlQuery, "QSqlQuery::exec: empty query");
         return false;
     }
 
     bool retval = d->sqlResult->reset(query);
 #ifdef QT_DEBUG_SQL
-    qDebug().nospace() << "Executed query (" << t.elapsed() << "ms, " << d->sqlResult->size()
-                       << " results, " << d->sqlResult->numRowsAffected()
-                       << " affected): " << d->sqlResult->lastQuery();
+    qCDebug(lcSqlQuery()).nospace() << "Executed query (" << t.elapsed() << "ms, "
+                                    << d->sqlResult->size()
+                                    << " results, " << d->sqlResult->numRowsAffected()
+                                    << " affected): " << d->sqlResult->lastQuery();
 #endif
     return retval;
 }
@@ -432,7 +440,7 @@ QVariant QSqlQuery::value(int index) const
 {
     if (isActive() && isValid() && (index > -1))
         return d->sqlResult->data(index);
-    qWarning("QSqlQuery::value: not positioned on a valid record");
+    qCWarning(lcSqlQuery, "QSqlQuery::value: not positioned on a valid record");
     return QVariant();
 }
 
@@ -443,14 +451,16 @@ QVariant QSqlQuery::value(int index) const
     If field \a name does not exist an invalid variant is returned.
 
     This overload is less efficient than \l{QSqlQuery::}{value()}
-*/
 
-QVariant QSqlQuery::value(const QString& name) const
+    \note In Qt versions prior to 6.8, this function took QString, not
+    QAnyStringView.
+*/
+QVariant QSqlQuery::value(QAnyStringView name) const
 {
     qsizetype index = d->sqlResult->record().indexOf(name);
     if (index > -1)
         return value(index);
-    qWarning("QSqlQuery::value: unknown field name '%s'", qPrintable(name));
+    qCWarning(lcSqlQuery, "QSqlQuery::value: unknown field name '%ls'", qUtf16Printable(name.toString()));
     return QVariant();
 }
 
@@ -596,7 +606,7 @@ bool QSqlQuery::seek(int index, bool relative)
     }
     // let drivers optimize
     if (isForwardOnly() && actualIdx < at()) {
-        qWarning("QSqlQuery::seek: cannot seek backwards in a forward only query");
+        qCWarning(lcSqlQuery, "QSqlQuery::seek: cannot seek backwards in a forward only query");
         return false;
     }
     if (actualIdx == (at() + 1) && at() != QSql::BeforeFirstRow) {
@@ -702,7 +712,7 @@ bool QSqlQuery::previous()
     if (!isSelect() || !isActive())
         return false;
     if (isForwardOnly()) {
-        qWarning("QSqlQuery::seek: cannot seek backwards in a forward only query");
+        qCWarning(lcSqlQuery, "QSqlQuery::seek: cannot seek backwards in a forward only query");
         return false;
     }
 
@@ -735,7 +745,7 @@ bool QSqlQuery::first()
     if (!isSelect() || !isActive())
         return false;
     if (isForwardOnly() && at() > QSql::BeforeFirstRow) {
-        qWarning("QSqlQuery::seek: cannot seek backwards in a forward only query");
+        qCWarning(lcSqlQuery, "QSqlQuery::seek: cannot seek backwards in a forward only query");
         return false;
     }
     return d->sqlResult->fetchFirst();
@@ -850,10 +860,9 @@ bool QSqlQuery::isSelect() const
 }
 
 /*!
-  Returns \c true if you can only scroll forward through a result set;
-  otherwise returns \c false.
+  Returns \l forwardOnly.
 
-  \sa setForwardOnly(), next()
+  \sa forwardOnly, next(), seek()
 */
 bool QSqlQuery::isForwardOnly() const
 {
@@ -861,7 +870,10 @@ bool QSqlQuery::isForwardOnly() const
 }
 
 /*!
-  Sets forward only mode to \a forward. If \a forward is true, only
+  \property QSqlQuery::forwardOnly
+  \since 6.8
+
+  This property holds the forward only mode. If \a forward is true, only
   next() and seek() with positive values, are allowed for navigating
   the results.
 
@@ -890,7 +902,11 @@ bool QSqlQuery::isForwardOnly() const
   mode, do not execute any other SQL command on the same database
   connection. This will cause the query results to be lost.
 
-  \sa isForwardOnly(), next(), seek(), QSqlResult::setForwardOnly()
+  \sa next(), seek()
+*/
+/*!
+    Sets \l forwardOnly to \a forward.
+    \sa forwardOnly, next(), seek()
 */
 void QSqlQuery::setForwardOnly(bool forward)
 {
@@ -976,19 +992,19 @@ bool QSqlQuery::prepare(const QString& query)
         d->sqlResult->setNumericalPrecisionPolicy(d->sqlResult->numericalPrecisionPolicy());
     }
     if (!driver()) {
-        qWarning("QSqlQuery::prepare: no driver");
+        qCWarning(lcSqlQuery, "QSqlQuery::prepare: no driver");
         return false;
     }
     if (!driver()->isOpen() || driver()->isOpenError()) {
-        qWarning("QSqlQuery::prepare: database not open");
+        qCWarning(lcSqlQuery, "QSqlQuery::prepare: database not open");
         return false;
     }
     if (query.isEmpty()) {
-        qWarning("QSqlQuery::prepare: empty query");
+        qCWarning(lcSqlQuery, "QSqlQuery::prepare: empty query");
         return false;
     }
 #ifdef QT_DEBUG_SQL
-    qDebug("\n QSqlQuery::prepare: %s", query.toLocal8Bit().constData());
+    qCDebug(lcSqlQuery, "\n QSqlQuery::prepare: %ls", qUtf16Printable(query));
 #endif
     return d->sqlResult->savePrepare(query);
 }
@@ -1015,9 +1031,9 @@ bool QSqlQuery::exec()
 
     bool retval = d->sqlResult->exec();
 #ifdef QT_DEBUG_SQL
-    qDebug().nospace() << "Executed prepared query (" << t.elapsed() << "ms, "
-                       << d->sqlResult->size() << " results, " << d->sqlResult->numRowsAffected()
-                       << " affected): " << d->sqlResult->lastQuery();
+    qCDebug(lcSqlQuery).nospace() << "Executed prepared query (" << t.elapsed() << "ms, "
+                                  << d->sqlResult->size() << " results, " << d->sqlResult->numRowsAffected()
+                                  << " affected): " << d->sqlResult->lastQuery();
 #endif
     return retval;
 }
@@ -1029,8 +1045,6 @@ bool QSqlQuery::exec()
 */
 
 /*!
-    \since 4.2
-
   Executes a previously prepared SQL query in a batch. All the bound
   parameters have to be lists of variants. If the database doesn't
   support batch executions, the driver will simulate it using
@@ -1234,6 +1248,8 @@ QVariant QSqlQuery::lastInsertId() const
 }
 
 /*!
+  \property QSqlQuery::numericalPrecisionPolicy
+  \since 6.8
 
   Instruct the database driver to return numerical values with a
   precision specified by \a precisionPolicy.
@@ -1252,17 +1268,19 @@ QVariant QSqlQuery::lastInsertId() const
   active query. Call \l{exec()}{exec(QString)} or prepare() in order
   to activate the policy.
 
-  \sa QSql::NumericalPrecisionPolicy, numericalPrecisionPolicy()
+  \sa QSql::NumericalPrecisionPolicy, QSqlDriver::numericalPrecisionPolicy,
+  QSqlDatabase::numericalPrecisionPolicy
 */
+/*!
+    Sets \l numericalPrecisionPolicy to \a precisionPolicy.
+ */
 void QSqlQuery::setNumericalPrecisionPolicy(QSql::NumericalPrecisionPolicy precisionPolicy)
 {
     d->sqlResult->setNumericalPrecisionPolicy(precisionPolicy);
 }
 
 /*!
-  Returns the current precision policy.
-
-  \sa QSql::NumericalPrecisionPolicy, setNumericalPrecisionPolicy()
+    Returns the \l numericalPrecisionPolicy.
 */
 QSql::NumericalPrecisionPolicy QSqlQuery::numericalPrecisionPolicy() const
 {
@@ -1270,18 +1288,23 @@ QSql::NumericalPrecisionPolicy QSqlQuery::numericalPrecisionPolicy() const
 }
 
 /*!
-  Enables or disables the positional \l {Approaches to Binding Values}{binding}
+  \property QSqlQuery::positionalBindingEnabled
+  \since 6.8
+  This property enables or disables the positional \l {Approaches to Binding Values}{binding}
   for this query, depending on \a enable (default is \c true).
   Disabling positional bindings is useful if the query itself contains a '?'
   which must not be handled as a positional binding parameter but, for example,
   as a JSON operator for a PostgreSQL database.
 
-  This function will have no effect when the database has native
+  This property will have no effect when the database has native
   support for positional bindings with question marks (see also
   \l{QSqlDriver::PositionalPlaceholders}).
+*/
 
+/*!
+  Sets \l positionalBindingEnabled to \a enable.
   \since 6.7
-  \sa isPositionalBindingEnabled()
+  \sa positionalBindingEnabled
 */
 void QSqlQuery::setPositionalBindingEnabled(bool enable)
 {
@@ -1289,10 +1312,9 @@ void QSqlQuery::setPositionalBindingEnabled(bool enable)
 }
 
 /*!
-  Returns \c true if the positional binding is currently enabled.
-
+  Returns \l positionalBindingEnabled.
   \since 6.7
-  \sa setPositionalBindingEnabled()
+  \sa positionalBindingEnabled
 */
 bool QSqlQuery::isPositionalBindingEnabled() const
 {
@@ -1301,8 +1323,6 @@ bool QSqlQuery::isPositionalBindingEnabled() const
 
 
 /*!
-  \since 4.3.2
-
   Instruct the database driver that no more data will be fetched from
   this query until it is re-executed. There is normally no need to
   call this function, but it may be helpful in order to free resources
@@ -1324,8 +1344,6 @@ void QSqlQuery::finish()
 }
 
 /*!
-  \since 4.4
-
   Discards the current result set and navigates to the next if available.
 
   Some databases are capable of returning multiple result sets for
@@ -1351,7 +1369,7 @@ void QSqlQuery::finish()
   databases may have restrictions on which statements are allowed to
   be used in a SQL batch.
 
-  \sa QSqlDriver::hasFeature(), setForwardOnly(), next(), isSelect(),
+  \sa QSqlDriver::hasFeature(), forwardOnly, next(), isSelect(),
       numRowsAffected(), isActive(), lastError()
 */
 bool QSqlQuery::nextResult()
